from .chart import chart
from .decode import decode
from .encode import encode

"""
Welcome to MorseCodePy, a module that helps you using Morse code in Python.

For detailed documentation, visit the GitHub repository:
https://github.com/CrazyFlyKite/MorseCodePy?tab=readme-ov-file#usage
"""
